@@ -118,7 +118,7 @@ void search_init(bool replacing, bool retain_answer)
 			if (*answer != '\0') {
 				last_search = mallocstrcpy(last_search, answer);
 #ifdef ENABLE_HISTORIES
-				update_history(&search_history, answer);
+				update_history(&search_history, answer, PRUNE_DUPLICATE);
 #endif
 			}
 
@@ -152,7 +152,7 @@ void search_init(bool replacing, bool retain_answer)
 			} else
 				replacing = !replacing;
 		} else if (func == flip_goto) {
-			do_gotolinecolumn(openfile->current->lineno,
+			goto_line_and_column(openfile->current->lineno,
 								openfile->placewewant + 1, TRUE, TRUE);
 			break;
 		} else
@@ -327,9 +327,7 @@ int findnextstr(const char *needle, bool whole_word_only, int modus,
 		spotlighted = TRUE;
 		light_from_col = xplustabs();
 		light_to_col = wideness(line->data, found_x + found_len);
-		if (!ISSET(SHOW_CURSOR))
-			hide_cursor = TRUE;
-		edit_refresh();
+		refresh_needed = TRUE;
 	}
 #endif
 
@@ -550,7 +548,7 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 			openfile->current_x = bot_x;
 		}
 	}
-#endif /* !NANO_TINY */
+#endif
 
 	came_full_circle = FALSE;
 
@@ -606,17 +604,16 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 		}
 
 		if (choice == 1 || replaceall) {  /* Yes, replace it. */
-			char *copy;
 			size_t length_change;
+			char *altered;
+
+			altered = replace_line(needle);
+
+			length_change = strlen(altered) - strlen(openfile->current->data);
 
 #ifndef NANO_TINY
 			add_undo(REPLACE, NULL);
-#endif
-			copy = replace_line(needle);
 
-			length_change = strlen(copy) - strlen(openfile->current->data);
-
-#ifndef NANO_TINY
 			/* If the mark was on and it was located after the cursor,
 			 * then adjust its x position for any text length changes. */
 			if (was_mark && !right_side_up) {
@@ -632,8 +629,9 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
 			/* If the mark was not on or it was before the cursor, then
 			 * adjust the cursor's x position for any text length changes. */
-			if (!was_mark || right_side_up) {
+			if (!was_mark || right_side_up)
 #endif
+			{
 				if (openfile->current == real_current &&
 						openfile->current_x < *real_current_x) {
 					if (*real_current_x < openfile->current_x + match_len)
@@ -641,8 +639,8 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 					*real_current_x += length_change;
 #ifndef NANO_TINY
 					bot_x = *real_current_x;
-				}
 #endif
+				}
 			}
 
 			/* Don't find the same zero-length or BOL match again. */
@@ -655,9 +653,9 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 				openfile->current_x += match_len + length_change;
 
 			/* Update the file size, and put the changed line into place. */
-			openfile->totsize += mbstrlen(copy) - mbstrlen(openfile->current->data);
+			openfile->totsize += mbstrlen(altered) - mbstrlen(openfile->current->data);
 			free(openfile->current->data);
-			openfile->current->data = copy;
+			openfile->current->data = altered;
 
 			set_modified();
 			as_an_at = TRUE;
@@ -706,7 +704,7 @@ void ask_for_and_do_replacements(void)
 #ifdef ENABLE_HISTORIES
 	/* When not "", add the replace string to the replace history list. */
 	if (response == 0)
-		update_history(&replace_history, answer);
+		update_history(&replace_history, answer, PRUNE_DUPLICATE);
 #endif
 
 	/* When cancelled, or when a function was run, get out. */
@@ -724,7 +722,7 @@ void ask_for_and_do_replacements(void)
 	openfile->current = beginline;
 	openfile->current_x = begin_x;
 
-	edit_refresh();
+	refresh_needed = TRUE;
 
 	if (numreplaced >= 0)
 		statusline(REMARK, P_("Replaced %zd occurrence",
@@ -747,8 +745,8 @@ void goto_line_posx(ssize_t line, size_t pos_x)
 /* Go to the specified line and column, or ask for them if interactive
  * is TRUE.  In the latter case also update the screen afterwards.
  * Note that both the line and column number should be one-based. */
-void do_gotolinecolumn(ssize_t line, ssize_t column, bool retain_answer,
-		bool interactive)
+void goto_line_and_column(ssize_t line, ssize_t column, bool retain_answer,
+							bool interactive)
 {
 	if (interactive) {
 		/* Ask for the line and column. */
@@ -844,9 +842,9 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool retain_answer,
 }
 
 /* Go to the specified line and column, asking for them beforehand. */
-void do_gotolinecolumn_void(void)
+void do_gotolinecolumn(void)
 {
-	do_gotolinecolumn(openfile->current->lineno,
+	goto_line_and_column(openfile->current->lineno,
 						openfile->placewewant + 1, FALSE, TRUE);
 }
 
